#define PICKER_VIEW_OFFSET  0
#define PICKER_VIEW_LENGTH  1

#define ALERT_VIEW_MASK     0

#import "EpcMaskEditViewController.h"
#import "AppDelegate.h"
#import "SystemSetting.h"
@interface EpcMaskEditViewController ()<AsReaderP252BManagerDelegate>

@property (weak, nonatomic) IBOutlet UITextField *offsetTextField;
@property (weak, nonatomic) IBOutlet UITextField *maskTextField;
@property (weak, nonatomic) IBOutlet UITextField *lengthTextField;
@property (strong, nonatomic) NSArray *offsetPickerData;
@property (strong, nonatomic) NSArray *lengthPickerData;
@property (strong, nonatomic) NSArray *bitData;
@property (weak, nonatomic) IBOutlet UIPickerView *pickerView;
@property (weak, nonatomic) IBOutlet UIView *offsetPickerTopView;
@property (weak, nonatomic) IBOutlet UIView *lengthPickerTopView;
@property (strong, nonatomic) AsReaderP252BSelectMaskEPCParam *selectMaskParam;
@end

@implementation EpcMaskEditViewController{
    AsReaderP252BManager *p252bManager;
    SystemSetting *systemSetting;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    systemSetting = [SystemSetting sharedSystemSetting];
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    int count = [p252bManager.asReaderP252B epcMaskCount];
    if (count > 0) {
        self.selectMaskParam = [p252bManager.asReaderP252B getEpcMask:0];
        if (self.selectMaskParam != nil) {
            [self setOffset:[self getBitDataIndex:self.selectMaskParam.offset]];
            [self setMask:self.selectMaskParam.mask];
            if (self.selectMaskParam.length == 0) {
                self.selectMaskParam.length = 16;
            }
            [self setLength:[self getBitDataIndex:self.selectMaskParam.length]];
        } else {
            self.selectMaskParam = [[AsReaderP252BSelectMaskEPCParam alloc] init];
            [self setOffset:1];
            [self setLength:1];
        }
    } else {
        self.selectMaskParam = [[AsReaderP252BSelectMaskEPCParam alloc] init];
        [self setOffset:1];
        [self setLength:1];
    }
}
- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    p252bManager.delegate = self;
    if ([self.navigationController respondsToSelector:@selector(interactivePopGestureRecognizer)]) {
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    }
}
- (void)viewWillDisappear:(BOOL)animated{
    [super viewWillDisappear:animated];
    p252bManager.delegate = nil;
}
- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}

- (BOOL)shouldPerformSegueWithIdentifier:(NSString *)identifier sender:(id)sender {
    if ([identifier isEqualToString:@"BackToSource"]) {
        NSInteger maskBit = [[self.bitData objectAtIndex:[self.lengthPickerData indexOfObject:self.lengthTextField.text]] integerValue];
        
        if ([self.maskTextField.text length] != maskBit / 4) {
            UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Mask length is not valid" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
            [alert show];
            return NO;
        }
    }
    return YES;
}

#pragma mark - UIPickerViewDelegate
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_OFFSET) {
        return self.bitData.count;
    } else if (pickerView.tag == PICKER_VIEW_LENGTH) {
        return self.lengthPickerData.count;
    }
    return 0;
}
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_OFFSET) {
        return [self.offsetPickerData objectAtIndex:row];
    } else if (pickerView.tag == PICKER_VIEW_LENGTH) {
        return [self.lengthPickerData objectAtIndex:row];
    }
    return @"None";
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (alertView.tag == ALERT_VIEW_MASK) {
        if (buttonIndex == 1) {
            UITextField *textFiled = [alertView textFieldAtIndex:0];
            NSInteger maskBit = [[self.bitData objectAtIndex:[self.lengthPickerData indexOfObject:self.lengthTextField.text]] integerValue];
            if ([textFiled.text length] != maskBit / 4) {
                UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Mask length is not valid" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
                [alert show];
                return;
            }
            [self setMask:[alertView textFieldAtIndex:0].text];
        } else {
            [self setMask:@""];
        }
    }
}

#pragma mark - UITextFieldDelegate
- (BOOL)textField:(UITextField *)textField shouldChangeCharactersInRange:(NSRange)range replacementString:(NSString *)string {
    
    NSCharacterSet* nonHex = [[NSCharacterSet
                               characterSetWithCharactersInString: @"0123456789ABCDEFabcdef"]
                              invertedSet];
    NSRange nonHexRange = [string rangeOfCharacterFromSet:nonHex];
    BOOL isHex = (nonHexRange.location == NSNotFound);
    
    if (isHex) {
        NSString *newString = [textField.text stringByReplacingCharactersInRange:range withString:string];
        textField.text = [newString uppercaseString];
    }
    return NO;
}

#pragma mark - Getters
- (NSArray *)offsetPickerData {
    if (_offsetPickerData == nil) {
        NSMutableArray *tmpArray = [[NSMutableArray alloc] init];
        for (NSString *offset in self.bitData) {
            [tmpArray addObject:[NSString stringWithFormat:@"%@ bit", offset]];
        }
        _offsetPickerData = tmpArray;
    }
    return _offsetPickerData;
}
- (NSArray *)lengthPickerData {
    if (_lengthPickerData == nil) {
        NSMutableArray *tmpArray = [[NSMutableArray alloc] init];
        for (NSString *offset in self.bitData) {
            [tmpArray addObject:[NSString stringWithFormat:@"%@ bit", offset]];
        }
        _lengthPickerData = tmpArray;
    }
    return _lengthPickerData;
}

- (NSArray *)bitData {
    if (_bitData == nil) {
        _bitData = DATA_BIT;
    }
    return _bitData;
}


#pragma mark - IBAction
- (IBAction)offsetBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_OFFSET;
    [self.pickerView reloadAllComponents];
    
    if (![self.offsetTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.offsetPickerData indexOfObject:self.offsetTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.pickerView.hidden = NO;
    self.offsetPickerTopView.hidden = NO;
}

- (IBAction)maskBtnTapped:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Mask:" message:@"Please enter mask value." delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeAlphabet;
    [alert textFieldAtIndex:0].delegate = self;
    [alert textFieldAtIndex:0].text = self.maskTextField.text;
    alert.tag = ALERT_VIEW_MASK;
    
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}

- (IBAction)lengthBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_LENGTH;
    [self.pickerView reloadAllComponents];
    
    if (![self.lengthTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.lengthPickerData indexOfObject:self.lengthTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.pickerView.hidden = NO;
    self.lengthPickerTopView.hidden = NO;
}

- (IBAction)pickerCloseBtnTapped:(id)sender {
    [self hidePickerView];
}


- (IBAction)offsetPickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    [self setOffset:idx];
    [self hidePickerView];
}

- (IBAction)lengthPickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    [self setLength:idx];
    [self hidePickerView];
}

- (IBAction)onClickSaveButton:(UIButton *)sender {
    if(!self.selectMaskParam.mask || self.selectMaskParam.mask.length <= 0 ){
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Please input mask value." delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    NSInteger maskBit = [[self.bitData objectAtIndex:[self.lengthPickerData indexOfObject:self.lengthTextField.text]] integerValue];
    if ([self.maskTextField.text length] != maskBit / 4) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Error" message:@"Mask length is not valid" delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        return;
    }
    [AppDelegate showGlobalProgressHUDWithTitle:@"Saving..."];
//    [mrxc3Manager.mrxc3 clearEpcMask];
    [p252bManager.asReaderP252B clearEpcMask];
    [p252bManager.asReaderP252B addEpcMask:self.selectMaskParam];
    [systemSetting setSelectionEpcMaskData:[NSArray arrayWithObject:self.selectMaskParam]];
    [AppDelegate dismissGlobalHUD];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Success" message:@"Saved successfully" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
    [self.navigationController popViewControllerAnimated:YES];
}

- (IBAction)onClickClearButton:(UIButton *)sender {
    self.offsetTextField.text = [self.offsetPickerData objectAtIndex:1];
    self.maskTextField.text = @"";
    self.lengthTextField.text = [self.lengthPickerData objectAtIndex:1];
    self.selectMaskParam.offset = 16;
    self.selectMaskParam.length = 16;
    self.selectMaskParam.mask = @"";
    [p252bManager.asReaderP252B clearEpcMask];
    [systemSetting setSelectionEpcMaskData:nil];
}

#pragma mark - Custom
- (void) hidePickerView {
    self.offsetPickerTopView.hidden = YES;
    self.lengthPickerTopView.hidden = YES;
    self.pickerView.hidden = YES;
}

- (void)setOffset:(NSInteger)index {
    self.offsetTextField.text = [self.offsetPickerData objectAtIndex:index];
    self.selectMaskParam.offset = [[self.bitData objectAtIndex:index] intValue];
}

- (void)setMask:(NSString *)mask {
    self.maskTextField.text = mask;
    self.selectMaskParam.mask = mask;
}

- (void)setLength:(NSInteger)index {
    self.lengthTextField.text = [self.lengthPickerData objectAtIndex:index];
    self.selectMaskParam.length = [[self.bitData objectAtIndex:index] intValue];
}

- (NSInteger)getBitDataIndex:(NSInteger)bit {
    return [self.bitData indexOfObject:[NSString stringWithFormat:@"%ld", (long)bit]];
}
- (BOOL)onAsReaderP252BTriggerKeyEvent:(BOOL)status{
    return NO;
}
-(void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    if (!isConnected) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    }
}
 
@end
