//
//  FirmwareUpdateManager.m
//  AsReaderP252BDemo
//
//  Created by mac on 2020/10/28.
//  Copyright © 2020 zyl. All rights reserved.
//

#import "FirmwareUpdateManager.h"

#define PACKET_PREFIX                   0x7e          // '~'
#define PACKET_SUFFIX1                  0x0d          // '\r'
#define PACKET_SUFFIX2                  0x0a          // '\n'

#define PACKET                          256.0
@interface FirmwareUpdateManager ()
{
    NSMutableData *_mRecvData;
    BOOL updateFirmwareStatus;
    AsReaderP252BPacketManager * mPacket;
    AsReaderP252BManager *p252bManager;
}
@end
@implementation FirmwareUpdateManager

static FirmwareUpdateManager * sharedFirmwareUpdateManager;
+ (FirmwareUpdateManager *)sharedFirmwareUpdateManager{
    if (!sharedFirmwareUpdateManager) {
        sharedFirmwareUpdateManager = [[FirmwareUpdateManager alloc] init];
    }
    return sharedFirmwareUpdateManager;
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        updateFirmwareStatus = NO;
        mPacket = [[AsReaderP252BPacketManager alloc] init];
        p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    }
    return self;
}
- (void)updateFirmwareWithFileName:(NSString *)fileName
                          fileData:(NSData *)data
                            result:(void(^)(BOOL result))result
                          progress:(void(^)(NSProgress * progress))progress{
    @try {
        if (!data || data.length == 0 || data.length % 2 != 0) {
            result(NO);
            return;
        }
        
        if (!p252bManager.asReaderP252B.isConnected) {
            result(NO);
            return;
        }
        
        [self startUpdateFirmware];
        //获取地址
        NSString *address = [self getFirmwareAddress];
        if (!address || [address isEqualToString:@""]) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        NSData *sendFileData = [[NSData alloc] init];
        if([@"00008040000" isEqualToString:address]){
            //文件后半部分数据传输
            sendFileData = [data subdataWithRange:NSMakeRange(data.length/2, data.length/2)];
        }else if([@"00008060000" isEqualToString:address]){
            //文件前半部分数据传输
            sendFileData = [data subdataWithRange:NSMakeRange(0, data.length/2)];
        }else{
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        if (sendFileData.length == 0) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        NSString *fileSize = 0;
        fileSize = [NSString stringWithFormat:@"%lu",(unsigned long)sendFileData.length];
        int countPicket =  ceilf(sendFileData.length/PACKET);
        NSLog(@"countPicket : %d",countPicket);
        NSProgress * prog = [NSProgress progressWithTotalUnitCount:countPicket];

        //发送固件信息
        
        NSData *sendData = [@"W3" dataUsingEncoding:NSASCIIStringEncoding];
        BOOL updateFirmwareInfo = [self writeFirmwareData:sendData Property:P252BFirmwareTypePicketStart];
        if (updateFirmwareInfo == NO) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        int resend = 0;
        int picketNumber = -1;
        int resendNumber = 0;
        for (int i = 0 ; i < countPicket ; i++) {
            NSData * sendData = nil;
            if (sendFileData.length > i*PACKET + PACKET) {
                sendData = [sendFileData subdataWithRange:NSMakeRange( i*PACKET, PACKET)];
            }else{
                sendData = [sendFileData subdataWithRange:NSMakeRange( i*PACKET, sendFileData.length - i*PACKET)];
            }
            picketNumber ++;
            if (picketNumber > 255) {
                picketNumber = 0;
            }
            NSLog(@"picketNumber : %d",picketNumber);
            if (!p252bManager.asReaderP252B.isConnected) {
                NSLog(@"_W3 is not connected");
                result(NO);
                [self stopUpdateFirmware];
                return;
            }
            NSMutableData *sendPicketData = [[NSMutableData alloc] init];
            Byte picket[1] = {static_cast<Byte>(picketNumber)};
            [sendPicketData appendData:[NSData dataWithBytes:picket length:1]];
            [sendPicketData appendData:sendData];
            
            NSMutableData *newData = [[NSMutableData alloc] init];
            NSData *lenghtData = [self convertHexStrToData:[NSString stringWithFormat:@"%04x", (int)sendPicketData.length + 4]];
            [newData appendData:lenghtData];
            [newData appendData:sendPicketData];
            int crc = [self calculateCRC:newData];
            
            NSData *crcData = [self convertHexStrToData:[NSString stringWithFormat:@"%04x", crc]];
            [newData appendData:crcData];
            
            
            BOOL updateFirmwarePickety = [self writeFirmwareData:newData Property:P252BFirmwareTypePicket];
            [NSThread sleepForTimeInterval:0.05];
            if (updateFirmwarePickety == NO) {
                if (resendNumber != i) {
                    resendNumber = i;
                    resend = 0;
                }
                i--;
                picketNumber--;
                resend++;
                if (resend > 4) {
                    result(NO);
                    [self stopUpdateFirmware];
                    return;
                }
            } else {
                prog.completedUnitCount++;
                progress(prog);
            }
        }
        //固件更新结束
        BOOL updateFirmwarePicketEND = [self writeFirmwareData:nil Property:P252BFirmwareTypePicketEND];
        [NSThread sleepForTimeInterval:0.05];
        if (updateFirmwarePicketEND == NO) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        } else {
            [prog setCompletedUnitCount:countPicket];
            progress(prog);
            result(YES);
            [self stopUpdateFirmware];
        }
    } @catch (NSException *exception) {
        result(NO);
        [self stopUpdateFirmware];
    } @finally {
    }
}
- (void)rX:(NSData *)readData {
    if (updateFirmwareStatus) {
        if (readData && readData.length > 0) {
            [self readData:readData];
        }
    }
}
- (BOOL)tX:(NSData *)transmitData {
    if (!updateFirmwareStatus) {
        return NO;
    }
    [p252bManager.asReaderP252B tx:transmitData];
    return YES;
}
- (void)startUpdateFirmware {
    updateFirmwareStatus = YES;
}
- (void)stopUpdateFirmware {
    updateFirmwareStatus = NO;
}
- (NSString *)getFirmwareAddress {
    _mRecvData = [[NSMutableData alloc] init];
    NSString *firmwareAddress = @"~r<\r\n";
    NSData *sendData = [firmwareAddress dataUsingEncoding:NSASCIIStringEncoding];
    [self tX:sendData];
    AsReaderP252BPacket *packet = [[AsReaderP252BPacket alloc]initWithPacketType:AsReaderP252BPacketResGetProperty subType:P252BFirmwareTypeAddress];
    [self wite:packet];
    AsReaderP252BResultCode resCode = (AsReaderP252BResultCode)packet.errorCode;
    if (resCode != AsReaderP252BResultNoError) {
        return @"";
    }
    NSData *resData = packet.data;
    NSString *firmwareAddressString = [[NSString alloc] initWithData:resData encoding:NSASCIIStringEncoding];
    return firmwareAddressString;
}
- (BOOL)writeFirmwareData:(NSData *)data Property:(P252BFirmwareType)property {
    _mRecvData = [[NSMutableData alloc] init];
    NSString *propertyString = [NSString stringWithFormat:@"~w%c", property];
    NSString *endData = @"\r\n";
    NSMutableData *sendData = [[NSMutableData alloc] initWithData:[propertyString dataUsingEncoding:NSASCIIStringEncoding]];
    if (data) {
        [sendData appendData:data];
    }
    [sendData appendData:[endData dataUsingEncoding:NSASCIIStringEncoding]];
    BOOL tXResult = [self tX:sendData];
    if (!tXResult) {
        return NO;
    }
    AsReaderP252BPacket *packet = [[AsReaderP252BPacket alloc]initWithPacketType:AsReaderP252BPacketResSetProperty subType:property];

    [self wite:packet];
    AsReaderP252BResultCode resCode = (AsReaderP252BResultCode)packet.errorCode;
    if (resCode != AsReaderP252BResultNoError) {
        return NO;
    } else {
        return YES;
    }
}
- (void)wite:(AsReaderP252BPacket *)packet{
    [mPacket addPacket:packet];
    [NSThread detachNewThreadSelector:@selector(changePacket:) toTarget:self withObject:packet];
    while (!packet.data) {
        if (!p252bManager.asReaderP252B.isConnected) {
            break;
        }
        [NSThread sleepForTimeInterval:0.01];
        [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]];
    }
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changePacket:) object:packet];
    [mPacket removePacket:packet];

}

- (void)changePacket:(AsReaderP252BPacket *)packet{
    [NSThread sleepForTimeInterval:2.0];
    [self performSelectorOnMainThread:@selector(cPacket:) withObject:packet waitUntilDone:NO];
}

- (void)cPacket:(AsReaderP252BPacket *)packet{
    if (!packet.data) {
        packet.data = [[NSData alloc] init];
        packet.errorCode = AsReaderP252BResultOtherError;
    }
}
- (void)readData:(NSData *)data {
    [_mRecvData appendData:data];
    NSData *recvData = nil;
    NSRange range;
    while(true)
    {
        range = [AsReaderP252BDataUtil findDataWidthRange:_mRecvData withPrefix:PACKET_PREFIX withSuffix1:PACKET_SUFFIX1 withSuffix2:PACKET_SUFFIX2];
        if (range.location == NSNotFound && range.length == NSNotFound) {
            return;
        }
        if (range.location != NSNotFound && range.length != NSNotFound) {
            if (range.length < 6) {
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
                return;
            } else {
                recvData = [_mRecvData subdataWithRange:NSMakeRange(range.location + 1, range.length - 3)];
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            }
        } else if (range.location != NSNotFound && range.length == NSNotFound) {
            if (range.location > 0) {
                range.length = range.location - 1;
                range.location = 0;
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            }
            return;
        } else if (range.location == NSNotFound && range.length != NSNotFound) {
            range.location = 0;
            [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            return;
        } else {
            return;
        }

        if (recvData.length < 2) {
            return;
        }
        uint8_t *totalPack = (uint8_t *)[recvData bytes];
        uint8_t pack = totalPack[0];
        uint8_t subPack = totalPack[1];
        NSInteger errorCode = [self getErrorCode:recvData findPosition:2];
        NSData *value = nil;
        int offset = 2;
        if (recvData.length > 2) {
            value = [recvData subdataWithRange:NSMakeRange(offset, recvData.length - offset)];
        }
        switch (pack) {
            case P252BFirmwarePropertyTypeGet://R
                switch (subPack) {
                    case P252BFirmwareTypeAddress://a
                    {
                        AsReaderP252BPacket *packet = [mPacket findPacket:AsReaderP252BPacketResGetProperty subType:subPack];
                        if (packet != nil) {
                            packet.errorCode = errorCode;
                            packet.data = value;
                            [packet signal];
                        }
                    }
                        break;
                    default:
                        break;
                }
                break;
            case P252BFirmwarePropertyTypeSet:
                switch (subPack) {
                    case P252BFirmwareTypePicketStart:
                    case P252BFirmwareTypePicket:
                    case P252BFirmwareTypePicketEND:
                    {
                        AsReaderP252BPacket *packet = [mPacket findPacket:AsReaderP252BPacketResSetProperty subType:subPack];
                        if (packet != nil) {
                            packet.errorCode = errorCode;
                            packet.data = value;
                            [packet signal];
                        }
                    }
                        break;
                    default:
                       
                        break;
                }
                break;
            default:
                break;
               
        }
    }
}
- (NSInteger)getErrorCode:(NSData *)data findPosition:(int)position {
    int length = 4;
    if ( (int)[data length] - position < 4) {
        length = (int)[data length] - position;
    }
    NSRange range = NSMakeRange(position, length);
    NSData *value = [data subdataWithRange:range];
    NSString *convValue = [[NSString alloc]initWithData:value encoding:NSASCIIStringEncoding];
    unsigned res = 0;
    [[NSScanner scannerWithString:convValue]scanHexInt:&res];
    return res;
}
- (NSData *)convertHexStrToData:(NSString *)str {
    if (!str || [str length] == 0) {
        return nil;
    }
    NSMutableData *hexData = [[NSMutableData alloc] initWithCapacity:8];
    NSRange range;
    if ([str length] % 2 == 0) {
        range = NSMakeRange(0, 2);
    } else {
        range = NSMakeRange(0, 1);
    }
    for (NSInteger i = range.location; i < [str length]; i += 2) {
        unsigned int anInt;
        NSString *hexCharStr = [str substringWithRange:range];
        NSScanner *scanner = [[NSScanner alloc] initWithString:hexCharStr];
        [scanner scanHexInt:&anInt];
        NSData *entity = [[NSData alloc] initWithBytes:&anInt length:1];
        [hexData appendData:entity];
        range.location += range.length;
        range.length = 2;
    }
    return hexData;
}

int crc16tab[] = {
    0x0000, 0x1021, 0x2042, 0x3063, 0x4084, 0x50a5, 0x60c6, 0x70e7,
    0x8108, 0x9129, 0xa14a, 0xb16b, 0xc18c, 0xd1ad, 0xe1ce, 0xf1ef,
    0x1231, 0x0210, 0x3273, 0x2252, 0x52b5, 0x4294, 0x72f7, 0x62d6,
    0x9339, 0x8318, 0xb37b, 0xa35a, 0xd3bd, 0xc39c, 0xf3ff, 0xe3de,
    0x2462, 0x3443, 0x0420, 0x1401, 0x64e6, 0x74c7, 0x44a4, 0x5485,
    0xa56a, 0xb54b, 0x8528, 0x9509, 0xe5ee, 0xf5cf, 0xc5ac, 0xd58d,
    0x3653, 0x2672, 0x1611, 0x0630, 0x76d7, 0x66f6, 0x5695, 0x46b4,
    0xb75b, 0xa77a, 0x9719, 0x8738, 0xf7df, 0xe7fe, 0xd79d, 0xc7bc,
    0x48c4, 0x58e5, 0x6886, 0x78a7, 0x0840, 0x1861, 0x2802, 0x3823,
    0xc9cc, 0xd9ed, 0xe98e, 0xf9af, 0x8948, 0x9969, 0xa90a, 0xb92b,
    0x5af5, 0x4ad4, 0x7ab7, 0x6a96, 0x1a71, 0x0a50, 0x3a33, 0x2a12,
    0xdbfd, 0xcbdc, 0xfbbf, 0xeb9e, 0x9b79, 0x8b58, 0xbb3b, 0xab1a,
    0x6ca6, 0x7c87, 0x4ce4, 0x5cc5, 0x2c22, 0x3c03, 0x0c60, 0x1c41,
    0xedae, 0xfd8f, 0xcdec, 0xddcd, 0xad2a, 0xbd0b, 0x8d68, 0x9d49,
    0x7e97, 0x6eb6, 0x5ed5, 0x4ef4, 0x3e13, 0x2e32, 0x1e51, 0x0e70,
    0xff9f, 0xefbe, 0xdfdd, 0xcffc, 0xbf1b, 0xaf3a, 0x9f59, 0x8f78,
    0x9188, 0x81a9, 0xb1ca, 0xa1eb, 0xd10c, 0xc12d, 0xf14e, 0xe16f,
    0x1080, 0x00a1, 0x30c2, 0x20e3, 0x5004, 0x4025, 0x7046, 0x6067,
    0x83b9, 0x9398, 0xa3fb, 0xb3da, 0xc33d, 0xd31c, 0xe37f, 0xf35e,
    0x02b1, 0x1290, 0x22f3, 0x32d2, 0x4235, 0x5214, 0x6277, 0x7256,
    0xb5ea, 0xa5cb, 0x95a8, 0x8589, 0xf56e, 0xe54f, 0xd52c, 0xc50d,
    0x34e2, 0x24c3, 0x14a0, 0x0481, 0x7466, 0x6447, 0x5424, 0x4405,
    0xa7db, 0xb7fa, 0x8799, 0x97b8, 0xe75f, 0xf77e, 0xc71d, 0xd73c,
    0x26d3, 0x36f2, 0x0691, 0x16b0, 0x6657, 0x7676, 0x4615, 0x5634,
    0xd94c, 0xc96d, 0xf90e, 0xe92f, 0x99c8, 0x89e9, 0xb98a, 0xa9ab,
    0x5844, 0x4865, 0x7806, 0x6827, 0x18c0, 0x08e1, 0x3882, 0x28a3,
    0xcb7d, 0xdb5c, 0xeb3f, 0xfb1e, 0x8bf9, 0x9bd8, 0xabbb, 0xbb9a,
    0x4a75, 0x5a54, 0x6a37, 0x7a16, 0x0af1, 0x1ad0, 0x2ab3, 0x3a92,
    0xfd2e, 0xed0f, 0xdd6c, 0xcd4d, 0xbdaa, 0xad8b, 0x9de8, 0x8dc9,
    0x7c26, 0x6c07, 0x5c64, 0x4c45, 0x3ca2, 0x2c83, 0x1ce0, 0x0cc1,
    0xef1f, 0xff3e, 0xcf5d, 0xdf7c, 0xaf9b, 0xbfba, 0x8fd9, 0x9ff8,
    0x6e17, 0x7e36, 0x4e55, 0x5e74, 0x2e93, 0x3eb2, 0x0ed1, 0x1ef0
};
- (int)calculateCRC:(NSData *)packetData {
    Byte *packet = (Byte *)[packetData bytes];
    int i;
    int crc = 0xFFFF;
    for (i = 0; i < packetData.length; i++) {
        crc = ((crc << 8) ^ crc16tab[((crc >> 8) ^ packet[i]) & 0x00FF]) & 0xFFFF;
    }
    return crc;
}
@end
