//
//  HumitureVC.m
//  AsReaderLR
//
//  Created by mac on 2018/8/9.
//  Copyright © 2018年 asterisk. All rights reserved.
//

#import "HumitureVC.h"
#import "AppDelegate.h"
#import "InventoryTableViewCell.h"
#import "HumitureItem.h"
#import "SystemSetting.h"
#import "AsReaderP252BManager.h"

@interface HumitureVC () <AsReaderP252BManagerDelegate, UITableViewDataSource, UITableViewDelegate>
@property (strong, nonatomic) IBOutlet UITableView *tableView;
@property (strong, nonatomic) IBOutlet UIButton *readButton;
@property (strong, nonatomic) IBOutlet UIButton *clearButton;
@property (strong, nonatomic) IBOutlet UISwitch *RSSISwitch;
@property (weak, nonatomic) IBOutlet UISwitch *continuousModeSwitch;
@property (strong, nonatomic) IBOutletCollection(UIControl) NSArray *toDisableElems;

@end

@implementation HumitureVC{
    AppDelegate *appDelegate;
    NSMutableArray *_dataArray;
    NSMutableDictionary *_dataDictionary;
    BOOL temperatureBool;
    SystemSetting * systemSetting;
    AsReaderP252BManager*p252bManager;
    BOOL isCanScan;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    
    temperatureBool = YES;
    _dataArray = [[NSMutableArray alloc] init];
    _dataDictionary = [[NSMutableDictionary alloc] init];
    systemSetting = [SystemSetting sharedSystemSetting];
    p252bManager.asReaderP252B.rssiMode = [systemSetting getRssiMode];
    [self.RSSISwitch setOn:p252bManager.asReaderP252B.rssiMode];
    [self.continuousModeSwitch setOn:p252bManager.asReaderP252B.continuousMode];
    [_tableView setRowHeight:50];
}

- (void)viewWillAppear:(BOOL)animated{
    [super viewWillAppear:animated];
    p252bManager.delegate = self;
    if ([self.navigationController respondsToSelector:@selector(interactivePopGestureRecognizer)]) {
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    }
}
- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    p252bManager.delegate = nil;
    [p252bManager.asReaderP252B stop];
}

#pragma mark - UITableViewDelegate
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return _dataArray.count;
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    InventoryTableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"InventoryCell" forIndexPath:indexPath];
    
    HumitureItem *item = nil;
    item = [_dataArray objectAtIndex:indexPath.row];
    if (item != nil) {
        cell.tagLabel.text = item.tag;
        cell.countLabel.text = item.vauleStr;
        if (self.RSSISwitch.isOn) {
            cell.rssiLabel.text = [NSString stringWithFormat:@"%.1f dB", item.rssi];
            cell.phaseLabel.text = [NSString stringWithFormat:@"%d˚", (int)item.phase];
            cell.frequency.text =  [NSString stringWithFormat:@"%.2fMHz ",item.frequency];
        } else {
            cell.rssiLabel.text = @"";
            cell.phaseLabel.text = @"";
            cell.frequency.text = @"";
        }
    }
    cell.accessoryType = UITableViewCellAccessoryDisclosureIndicator;
    cell.selectionStyle = UITableViewCellSelectionStyleNone;
    return cell;
}

#pragma mark - Action
- (IBAction)segmentAction:(UISegmentedControl *)sender {
    if (0 == sender.selectedSegmentIndex) {
        temperatureBool = YES;
    } else {
        temperatureBool = NO;
    }
    [_dataArray removeAllObjects];
    [_dataDictionary removeAllObjects];
    [self.tableView reloadData];
}
- (IBAction)continuousModeValueChanged:(id)sender {
    p252bManager.asReaderP252B.continuousMode = [self.continuousModeSwitch isOn];
    [systemSetting setContinuousMode:[self.continuousModeSwitch isOn]];
}

- (IBAction)switchAction:(UISwitch *)sender {
    p252bManager.asReaderP252B.rssiMode = [self.RSSISwitch isOn];
    [systemSetting setRssiMode:[self.RSSISwitch isOn]];
}

- (IBAction)readAction:(id)sender {
    if ([p252bManager.asReaderP252B getAction] == AsReaderP252BCommandStop) {
        if (temperatureBool) {
            [p252bManager.asReaderP252B inventoryTemperatureTagWithType:AsReaderP252BTemperatureTagType_0];
        } else {
            [p252bManager.asReaderP252B inventoryHumidityTag];
        }
    } else {
        [p252bManager.asReaderP252B stop];
    }
}
- (IBAction)clearAction:(id)sender {
    [_dataArray removeAllObjects];
    [_dataDictionary removeAllObjects];
    [self.tableView reloadData];
}

- (IBAction)backTapped:(UIBarButtonItem *)sender {
    [p252bManager.asReaderP252B stop];
    [self.navigationController popViewControllerAnimated:YES];
}
 
- (void)readAsReaderP252BTemperatureTag:(NSString *)tag rssi:(float)rssi phase:(float)phase frequency:(float)frequency temperature:(float)temperature{
    if (_dataArray && _dataArray.count == 0) {
        HumitureItem *item = [[HumitureItem alloc] init];
        item.tag = tag;
        item.rssi = rssi;
        item.phase = phase;
        item.frequency = frequency;
        item.vauleStr = [NSString stringWithFormat:@"%.2f°C",temperature];
        [_dataDictionary setObject:item forKey:tag];
        [_dataArray addObject:item];
    } else if (_dataArray && _dataArray.count > 0) {
        HumitureItem *item = [_dataDictionary objectForKey:tag];
        if (item == nil) {
            HumitureItem *item = [[HumitureItem alloc] init];
            item.tag = tag;
            item.rssi = rssi;
            item.phase = phase;
            item.frequency = frequency;
            item.vauleStr = [NSString stringWithFormat:@"%.2f°C",temperature];;
            [_dataDictionary setObject:item forKey:tag];
            [_dataArray addObject:item];
        } else {
            item.tag = tag;
            item.rssi = rssi;
            item.phase = phase;
            item.frequency = frequency;
            item.vauleStr = [NSString stringWithFormat:@"%.2f°C",temperature];;
        }
    }
    if (_dataArray && _dataArray.count != 0) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.tableView reloadData];
        });
    }
}

- (void)readAsReaderP252BHumidityTag:(NSString *)tag rssi:(float)rssi phase:(float)phase frequency:(float)frequency humidity:(float)humidity{
    if (_dataArray && _dataArray.count == 0) {
        HumitureItem *item = [[HumitureItem alloc] init];
        item.tag = tag;
        item.rssi = rssi;
        item.phase = phase;
        item.frequency = frequency;
        item.vauleStr = [NSString stringWithFormat:@"%.0f",humidity];;
        [_dataDictionary setObject:item forKey:tag];
        [_dataArray addObject:item];

    } else if (_dataArray && _dataArray.count > 0) {
        HumitureItem *item = [_dataDictionary objectForKey:tag];
        if (item == nil) {
            HumitureItem *item = [[HumitureItem alloc] init];
            item.tag = tag;
            item.rssi = rssi;
            item.phase = phase;
            item.frequency = frequency;
            item.vauleStr = [NSString stringWithFormat:@"%.0f",humidity];
            [_dataDictionary setObject:item forKey:tag];
            [_dataArray addObject:item];
        } else {
            item.tag = tag;
            item.rssi = rssi;
            item.phase = phase;
            item.frequency = frequency;
            item.vauleStr = [NSString stringWithFormat:@"%.0f",humidity];
        }
    }
    dispatch_async(dispatch_get_main_queue(), ^{
        [self.tableView reloadData];
    });
}
- (void)changedAsReaderP252BActionState:(AsReaderP252BCommandType)action resultCode:(NSInteger)resultCode{
    NSLog(@"changedActionState ： %02X",action);
    if (action == AsReaderP252BCommandStop){
        [self.readButton setTitle:@"Inventory" forState:UIControlStateNormal];
        [[self toDisableElems]enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
            UIControl *control = obj;
            [control setEnabled:YES];
        }];
        self.tableView.userInteractionEnabled = YES;
        if (resultCode == AsReaderP252BResultNoError) {
            isCanScan = YES;
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changStatus) object:nil];
        }else{
            isCanScan = NO;
            [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changStatus) object:nil];
            [self performSelector:@selector(changStatus) withObject:nil afterDelay:1.0];
        }
    }else{
        if (resultCode == AsReaderP252BResultNoError)
        {
            [self.readButton setTitle:@"Stop" forState:UIControlStateNormal];
            [[self toDisableElems]enumerateObjectsUsingBlock:^(id  _Nonnull obj, NSUInteger idx, BOOL * _Nonnull stop) {
                UIControl *control = obj;
                [control setEnabled:NO];
            }];
            self.tableView.userInteractionEnabled = NO;
        }
    }
}
- (void)changStatus{
    isCanScan = YES;
}
- (BOOL)onAsReaderP252BTriggerKeyEvent:(BOOL)status {
    if (status) {
        [self readAction:nil];
    }
    return NO;
}
-(void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    if (!isConnected) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    }
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}


@end
