#define PICKER_VIEW_BUZZER          0
#define PICKER_VIEW_KEY_ACTION      1
#define PICKER_VIEW_MASK_TYPE       2
#define PICKER_VIEW_QValue          4
#define PICKER_VIEW_MaxQValue       6
#define PICKER_VIEW_MinQValue       7
#define PICKER_VIEW_Link_Profile               8
#define PICKER_VIEW_Default_Link_Profile       9
#define PICKER_VIEW_Baud_Rate                  10


#define ALERT_VIEW_IDLE_TIME        1
#define ALERT_VIEW_AUTO_OFF_TIME    2
#define ALERT_VIEW_BATTERY_INTERVAL 3
#define ALERT_VIEW_SLEEP_TIME       4
#define ALERT_VIEW_BARCODE_TIMEOUT  5

#import "OptionsViewController.h"
#import "SystemSetting.h" 
#import "AsReaderP252BManager.h"
@interface OptionsViewController ()<AsReaderP252BManagerDelegate> {
    SystemSetting * systemSetting;
    AsReaderP252BManager *p252bManager;
}
@property (weak, nonatomic) IBOutlet UIScrollView *scrollView;
@property (weak, nonatomic) IBOutlet UIButton *setLBTButton;

@property (weak, nonatomic) IBOutlet UIPickerView *pickerView;
@property (weak, nonatomic) IBOutlet UITableView *tableView;
@property (weak, nonatomic) IBOutlet UITextField *buzzerTextField;
@property (weak, nonatomic) IBOutlet UITextField *idleTimeTextField;
@property (weak, nonatomic) IBOutlet UITextField *autoOffTimeTextField;
@property (weak, nonatomic) IBOutlet UITextField *keyActionTextField;
@property (weak, nonatomic) IBOutlet UITextField *batteryIntervalTextField;
@property (weak, nonatomic) IBOutlet UITextField *maskTypeTextField;
@property (weak, nonatomic) IBOutlet UIView *buzzerPickerTopView;
@property (weak, nonatomic) IBOutlet UIView *keyActionPickerTopView;
@property (weak, nonatomic) IBOutlet UIView *maskTypePickerTopView;
@property (weak, nonatomic) IBOutlet UIView *lbtChannelTableTopView;
@property (weak, nonatomic) IBOutlet UIView *baudRateTopView;
@property (weak, nonatomic) IBOutlet UIView *shadowView;
@property (strong, nonatomic) NSArray *buzzerPickerData;
@property (strong, nonatomic) NSArray *keyActionPickerData;
@property (strong, nonatomic) NSArray *maskTypePickerData;
@property (strong, nonatomic) NSArray *lbtItemData;
@property (readwrite, nonatomic) AsReaderP252BBuzzerState buzzerValue;
@property (readwrite, nonatomic) NSInteger idleTimeValue;
@property (readwrite, nonatomic) NSInteger autoOffTimeValue;
@property (readwrite, nonatomic) NSInteger keyActionValue;
@property (strong, nonatomic) NSString *batteryIntervalValue;
@property (strong, nonatomic) NSMutableArray *mBaudRateList;

@property (weak, nonatomic) IBOutlet UITextField *qValueTextField;
@property (readwrite, nonatomic) NSInteger qValue;
@property (strong, nonatomic) NSArray *qValuePickerData;
@property (weak, nonatomic) IBOutlet UIView *qValuePickerTopView;

@property (weak, nonatomic) IBOutlet UITextField *minQValueTextField;
@property (readwrite, nonatomic) NSInteger minQValue;

@property (weak, nonatomic) IBOutlet UITextField *maxQValueTextField;
@property (readwrite, nonatomic) NSInteger maxQValue;


@property (strong, nonatomic) NSArray *linkProfilePickerData;
@property (readwrite, nonatomic) NSInteger linkProfileValue;
@property (weak, nonatomic) IBOutlet UITextField *linkProfileTextField;


@property (readwrite, nonatomic) NSInteger defaultLinkProfileValue;
@property (weak, nonatomic) IBOutlet UITextField *defaultLinkProfileTextField;
@property (weak, nonatomic) IBOutlet UITextField *sleepTimeTextField;
@property (readwrite, nonatomic) NSInteger sleepTimeValue;
@property (readwrite, nonatomic) NSInteger barcodeTimeoutValue;
@property (readwrite, nonatomic) NSInteger baudRateValue;

@property (weak, nonatomic) IBOutlet UIView *LinkProfileTableTopView;
@property (weak, nonatomic) IBOutlet UILabel *lbtLabel;

@property (weak, nonatomic) IBOutlet UISwitch *lbtSwitch;
@property (weak, nonatomic) IBOutlet UITextField *barcodeTimeoutTextField;
@property (weak, nonatomic) IBOutlet UIButton *barcodeTimeoutButton;

@property (weak, nonatomic) IBOutlet UITextField *baudRateTextField;
@property (weak, nonatomic) IBOutlet UIButton *baudRateButton;
@end

@implementation OptionsViewController{
    BOOL isJapan;
}

- (void)viewDidLoad {
    [super viewDidLoad];
    isJapan = NO;
    p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    systemSetting = [SystemSetting sharedSystemSetting];
    self.scrollView.contentSize = CGSizeMake(0, 1000);
    self.scrollView.showsVerticalScrollIndicator = NO;
}

- (void)viewWillAppear:(BOOL)animated {
    [super viewWillAppear:animated];
    p252bManager.delegate = self;
    if ([self.navigationController respondsToSelector:@selector(interactivePopGestureRecognizer)]) {
        self.navigationController.interactivePopGestureRecognizer.enabled = NO;
    }
}

- (void)viewDidAppear:(BOOL)animated{
    [AppDelegate showGlobalProgressHUDWithTitle:@"Loading..."];
    appDelegate = (AppDelegate *)[[UIApplication sharedApplication] delegate];
    
    if (p252bManager.asReaderP252B == nil) {
        [AppDelegate dismissGlobalHUD];
        return;
    }
    self.buzzerValue = p252bManager.asReaderP252B.buzzer;
    self.idleTimeValue = p252bManager.asReaderP252B.idleTime;
    self.autoOffTimeValue = p252bManager.asReaderP252B.autoOffTime;
    self.qValue = p252bManager.asReaderP252B.qValue;
    self.maxQValue = p252bManager.asReaderP252B.maxQ;
    self.minQValue = p252bManager.asReaderP252B.minQ;
    self.linkProfileValue = p252bManager.asReaderP252B.linkProfileValue;
    self.defaultLinkProfileValue = [systemSetting getDefaultLinkProfileValue];
    self.sleepTimeValue = p252bManager.asReaderP252B.sleepTime;
    self.barcodeTimeoutValue = [p252bManager.asReaderP252B getBarcodeTimeOut];
    self.mBaudRateList = [p252bManager.asReaderP252B getBaudRateList];
    self.baudRateValue = [p252bManager.asReaderP252B getBaudRate];
    isJapan = [[p252bManager.asReaderP252B regionName] hasPrefix:@"Japan"];
    _setLBTButton.hidden = YES;
    _lbtLabel.hidden = YES;
    _lbtSwitch.hidden = YES;
    if (isJapan) {
        _lbtLabel.hidden = NO;
        _lbtSwitch.hidden = NO;
        if ([p252bManager.asReaderP252B getFrequencyAutomatic]) {
            [_lbtSwitch setOn:YES];
        }else{
            [_lbtSwitch setOn:NO];
            _setLBTButton.hidden = NO;
            [self initLbtItemData];
        }
    }
    [self initViewData];
    [self.tableView reloadData];
    [self.navigationController.view addSubview:self.shadowView];
    [AppDelegate dismissGlobalHUD];
}

- (void)viewWillDisappear:(BOOL)animated {
    [super viewWillDisappear:animated];
    p252bManager.delegate = nil;
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
}


#pragma mark - UITableViewDataSource
- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView {
    return 1;
}

- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    return self.lbtItemData.count;
}

#pragma mark - UITableViewDelegate
- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:@"LbtCell" forIndexPath:indexPath];
    
    AsReaderP252BLbtItem *lbtItem = [self.lbtItemData objectAtIndex:indexPath.row];
    
    if (lbtItem.mIsUsed) {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
    }else{
        cell.accessoryType = UITableViewCellAccessoryNone;

    }
    
    cell.textLabel.text = lbtItem.frequency;
    
    return cell;
}

- (void)tableView:(UITableView *)tableView didSelectRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    AsReaderP252BLbtItem *lbtItem = [self.lbtItemData objectAtIndex:indexPath.row];
    if (cell.accessoryType == UITableViewCellAccessoryCheckmark) {
        cell.accessoryType = UITableViewCellAccessoryNone;
        lbtItem.mIsUsed = NO;
    } else {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
        lbtItem.mIsUsed = YES;
    }
    
}

-(void)tableView:(UITableView *)tableView didDeselectRowAtIndexPath:(NSIndexPath *)indexPath
{
    UITableViewCell *cell = [tableView cellForRowAtIndexPath:indexPath];
    AsReaderP252BLbtItem *lbtItem = [self.lbtItemData objectAtIndex:indexPath.row];
    if (cell.accessoryType == UITableViewCellAccessoryCheckmark) {
        cell.accessoryType = UITableViewCellAccessoryNone;
        lbtItem.mIsUsed = NO;
    } else {
        cell.accessoryType = UITableViewCellAccessoryCheckmark;
        lbtItem.mIsUsed = YES;
    }
}


#pragma mark - UIPickerViewDataSource
- (NSInteger)numberOfComponentsInPickerView:(UIPickerView *)pickerView
{
    return 1;
}
- (NSInteger)pickerView:(UIPickerView *)pickerView numberOfRowsInComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_BUZZER) {
        return self.buzzerPickerData.count;
    } else if (pickerView.tag == PICKER_VIEW_KEY_ACTION) {
        return self.keyActionPickerData.count;
    } else if (pickerView.tag == PICKER_VIEW_MASK_TYPE) {
        return self.maskTypePickerData.count;
    } else if(pickerView.tag == PICKER_VIEW_QValue){
        return self.qValuePickerData.count;
    } else if(pickerView.tag == PICKER_VIEW_MinQValue){
        return self.qValuePickerData.count;
    } else if(pickerView.tag == PICKER_VIEW_MaxQValue){
        return self.qValuePickerData.count;
    } else if(pickerView.tag == PICKER_VIEW_Default_Link_Profile){
        return self.linkProfilePickerData.count;
    } else if(pickerView.tag == PICKER_VIEW_Link_Profile){
        return self.linkProfilePickerData.count;
    } else if (pickerView.tag == PICKER_VIEW_Baud_Rate){
        return self.mBaudRateList.count;
    }
    return 0;
}

#pragma mark - UIPickerViewDelegate
- (NSString *)pickerView:(UIPickerView *)pickerView titleForRow:(NSInteger)row forComponent:(NSInteger)component
{
    if (pickerView.tag == PICKER_VIEW_BUZZER) {
        return [self.buzzerPickerData objectAtIndex:row];
    } else if (pickerView.tag == PICKER_VIEW_KEY_ACTION) {
        return [self.keyActionPickerData objectAtIndex:row];
    } else if (pickerView.tag == PICKER_VIEW_MASK_TYPE) {
        return [self.maskTypePickerData objectAtIndex:row];
    } else if(pickerView.tag == PICKER_VIEW_QValue){
        return [self.qValuePickerData objectAtIndex:row];
    } else if(pickerView.tag == PICKER_VIEW_MinQValue){
        return [self.qValuePickerData objectAtIndex:row];
    } else if(pickerView.tag == PICKER_VIEW_MaxQValue){
        return [self.qValuePickerData objectAtIndex:row];
    } else if(pickerView.tag == PICKER_VIEW_Default_Link_Profile){
        return [self.linkProfilePickerData objectAtIndex:row];
    } else if(pickerView.tag == PICKER_VIEW_Link_Profile){
        return [self.linkProfilePickerData objectAtIndex:row];
    } else if (pickerView.tag == PICKER_VIEW_Baud_Rate){
        return [self.mBaudRateList objectAtIndex:row];
    }
    return @"None";
}

#pragma mark - UIAlertViewDelegate
- (void)alertView:(UIAlertView *)alertView clickedButtonAtIndex:(NSInteger)buttonIndex
{
    if (alertView.tag == ALERT_VIEW_IDLE_TIME) {
        if (buttonIndex == 1) {
            NSString * value = [alertView textFieldAtIndex:0].text;
            if(value.length == 0){
                value = @"0";
            }
            if ([value intValue] > 65535) {
                value = @"65535";
            }
            [self setIdleTime:value];
        }
    } else if (alertView.tag == ALERT_VIEW_AUTO_OFF_TIME) {
        if (buttonIndex == 1) {
            NSString * value = [alertView textFieldAtIndex:0].text;
            if(value.length == 0){
                value = @"0";
            }
            if ([value intValue] > 1800) {
                value = @"1800";
            }
            [self setAutoOffTime:value];
        }
    } else if (alertView.tag == ALERT_VIEW_BATTERY_INTERVAL) {
        if (buttonIndex == 1) {
            NSString * value = [alertView textFieldAtIndex:0].text;
            if(value.length == 0){
                value = @"0";
            }
            if ([value intValue] > 1800) {
                value = @"1800";
            }
            [self setBatteryInterval:value];
        }
    } else if (alertView.tag == ALERT_VIEW_SLEEP_TIME) {
        if (buttonIndex == 1) {
            NSString * value = [alertView textFieldAtIndex:0].text;
            if(value.length == 0){
                value = @"0";
            }
            if ([value intValue] > 1800) {
                value = @"1800";
            }
            [self setSleepTime:value];
        }
    } else if (alertView.tag == ALERT_VIEW_BARCODE_TIMEOUT) {
        if (buttonIndex == 1) {
            NSString * value = [alertView textFieldAtIndex:0].text;
            if ([value intValue] < 4) {
                value = @"4";
            }
            if ([value intValue] > 300) {
                value = @"300";
            }
            [self setBarcodeTimeout:value];
        }
    }

}
#pragma mark - Getters
- (NSArray *)buzzerPickerData {
    return @[@"Off", @"Low", @"High"];
}
- (NSArray *)keyActionPickerData {
    return @[@"Not Used", @"Used"];
}
- (NSArray *)maskTypePickerData {
    return @[@"No Mask", @"EPC Mask"];
}
- (NSArray *)qValuePickerData {
    return @[@"00", @"01",@"02",@"03",@"04",@"05",@"06",@"07",@"08",@"09",@"10",@"11",@"12",@"13",@"14",@"15"];
}
- (NSArray *)linkProfilePickerData{
     return @[@"0",@"1",@"2",@"3"];
}

#pragma mark - Init
- (void)initLbtItemData {
    if (_lbtItemData == nil) {
        _lbtItemData = p252bManager.asReaderP252B.getFrequency;
    }
}

#pragma mark - IBAction
- (IBAction)minQValueBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_MinQValue;
    [self.pickerView reloadAllComponents];
    if (![self.minQValueTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.qValuePickerData indexOfObject:self.minQValueTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.qValuePickerTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)maxQValueBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_MaxQValue;
    [self.pickerView reloadAllComponents];
    if (![self.maxQValueTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.qValuePickerData indexOfObject:self.maxQValueTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.qValuePickerTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)qValueBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_QValue;
    [self.pickerView reloadAllComponents];
    if (![self.qValueTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.qValuePickerData indexOfObject:self.qValueTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.qValuePickerTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)linkProfileBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_Link_Profile;
    [self.pickerView reloadAllComponents];
    if (![self.linkProfileTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.linkProfilePickerData indexOfObject:self.linkProfileTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.LinkProfileTableTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)defaultLinkProfileBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_Default_Link_Profile;
    [self.pickerView reloadAllComponents];
    if (![self.defaultLinkProfileTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.linkProfilePickerData indexOfObject:self.defaultLinkProfileTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.LinkProfileTableTopView.hidden = NO;
    [self showPickerView];
}

- (IBAction)buzzerBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_BUZZER;
    [self.pickerView reloadAllComponents];
    
    if (![self.buzzerTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.buzzerPickerData indexOfObject:self.buzzerTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.buzzerPickerTopView.hidden = NO;
    [self showPickerView];
}

- (IBAction)idleTimeBtnTapped:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Idle Time:" message:@"Please enter idle time.\n(0~65535 *10ms)" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeNumberPad;
    alert.tag = ALERT_VIEW_IDLE_TIME;
    
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}
- (IBAction)autoOffTimeBtnTapped:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Auto Off Time:" message:@"Please enter auto off time.\n(0~1800s)" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeNumberPad;
    alert.tag = ALERT_VIEW_AUTO_OFF_TIME;
    
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}
- (IBAction)keyActionBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_KEY_ACTION;
    [self.pickerView reloadAllComponents];
    
    if (![self.keyActionTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.keyActionPickerData indexOfObject:self.keyActionTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.keyActionPickerTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)batteryIntervalBtnTapped:(id)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Battery Interval:" message:@"Please enter battery interval.\n(0~1800s)" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeNumberPad;
    alert.tag = ALERT_VIEW_BATTERY_INTERVAL;
    
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}
- (IBAction)maskTypeBtnTapped:(id)sender {
    self.pickerView.tag = PICKER_VIEW_MASK_TYPE;
    [self.pickerView reloadAllComponents];
    
    if (![self.maskTypeTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.maskTypePickerData indexOfObject:self.maskTypeTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    
    self.maskTypePickerTopView.hidden = NO;
    [self showPickerView];
}
- (IBAction)lbtAutomaticSwitch:(id)sender {
    [p252bManager.asReaderP252B setFrequencyAutomatic:_lbtSwitch.isOn];
    [systemSetting setFrequencyAutomatic:_lbtSwitch.isOn];
    if(_lbtSwitch.isOn){
        _setLBTButton.hidden = YES;
    }else{
        _setLBTButton.hidden = NO;
        _lbtItemData = p252bManager.asReaderP252B.getFrequency;
    }
    
}

- (IBAction)sleepTimeBtnTapped:(id)sender{
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"sleep Time:" message:@"Please enter sleep time.\n(0~1800s)" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeNumberPad;
    alert.tag = ALERT_VIEW_SLEEP_TIME;
    
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}
- (IBAction)saveBtnTapped:(id)sender {
    [AppDelegate showGlobalProgressHUDWithTitle:@"Saving..."];
    p252bManager.asReaderP252B.buzzer = self.buzzerValue;
    p252bManager.asReaderP252B.idleTime = (int)self.idleTimeValue;
    p252bManager.asReaderP252B.autoOffTime = (int)self.autoOffTimeValue;
    p252bManager.asReaderP252B.qValue = (int)self.qValue;
    p252bManager.asReaderP252B.sleepTime = (int) self.sleepTimeValue;
    [p252bManager.asReaderP252B setBarcodeTimeOut:(int)self.barcodeTimeoutValue];
    [p252bManager.asReaderP252B setBaudRate:(int)self.baudRateValue];
    [[SystemSetting sharedSystemSetting] setBatteryInterval:self.batteryIntervalValue];
    
    [systemSetting setBuzzer:self.buzzerValue];
    [systemSetting setIdleTime:(int)self.idleTimeValue ];
    [systemSetting setAutoOffTime:(int)self.autoOffTimeValue ];
    [systemSetting setQValue:(int)self.qValue];
    [systemSetting setSleepTime:(int) self.sleepTimeValue ];
    [systemSetting setBarcodeTimeout:(int)self.barcodeTimeoutValue];
    [systemSetting setBaudRate:(int)self.baudRateValue];

    [AppDelegate dismissGlobalHUD];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Success" message:@"Saved successfully" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}
- (IBAction)defaultBtnTapped:(id)sender {
    [AppDelegate showGlobalProgressHUDWithTitle:@"Saving..."];
    [systemSetting setDefaultLinkProfileValue:1];
    AsReaderP252BResultCode res = p252bManager.asReaderP252B.defaultParameter;
    if (res != AsReaderP252BResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Fail" message:@"Set to default failed" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
        
        [alert show];
        return;
    }
    
    self.buzzerValue = p252bManager.asReaderP252B.buzzer;
    self.idleTimeValue = p252bManager.asReaderP252B.idleTime;
    self.autoOffTimeValue = p252bManager.asReaderP252B.autoOffTime;
    self.qValue = p252bManager.asReaderP252B.qValue;
    self.maxQValue = p252bManager.asReaderP252B.maxQ;
    self.minQValue = p252bManager.asReaderP252B.minQ;
    self.sleepTimeValue = p252bManager.asReaderP252B.sleepTime;
    self.barcodeTimeoutValue = [p252bManager.asReaderP252B getBarcodeTimeOut];
    self.linkProfileValue = p252bManager.asReaderP252B.linkProfileValue;
    self.defaultLinkProfileValue = [systemSetting getDefaultLinkProfileValue];
    BOOL sta = p252bManager.asReaderP252B.getFrequencyAutomatic;
    [systemSetting setFrequencyAutomatic:sta];
    if (sta)
    {
        [systemSetting setLbtItemData:[NSArray array]];
        [_lbtSwitch setOn:YES];
        _setLBTButton.hidden = YES;
    }
    self.baudRateValue = [p252bManager.asReaderP252B getBaudRate];
    [systemSetting setBuzzer:self.buzzerValue];
    [systemSetting setIdleTime:(int)self.idleTimeValue ];
    [systemSetting setAutoOffTime:(int)self.autoOffTimeValue ];
    [systemSetting setQValue:(int)self.qValue];
    [systemSetting setSleepTime:(int) self.sleepTimeValue ];
    [systemSetting setBarcodeTimeout:(int)self.barcodeTimeoutValue];
    [systemSetting setBaudRate:(int)self.baudRateValue];
    [systemSetting setSessionFlag:p252bManager.asReaderP252B.sessionFlag];
    [systemSetting setInventorySession:p252bManager.asReaderP252B.inventorySession];
    [systemSetting setOperationTime:p252bManager.asReaderP252B.operationTime];
    [systemSetting setPowerGain:300];
    
   
    
    [[SystemSetting sharedSystemSetting] setBatteryInterval:@"10"];
    [self initViewData];
    [AppDelegate dismissGlobalHUD];
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Success" message:@"Set to default successfully" delegate:self cancelButtonTitle:@"OK" otherButtonTitles:nil];
    [alert show];
}
- (IBAction)pickerCloseBtnTapped:(id)sender {
    [self hidePickerView];
}

- (IBAction)qValuePickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    if (self.pickerView.tag == PICKER_VIEW_MaxQValue ) {
        [self setMaxQ:idx];
    } else if (self.pickerView.tag == PICKER_VIEW_MinQValue ) {
        [self setMinQ:idx];
    } else if (self.pickerView.tag == PICKER_VIEW_QValue ) {
        [self setQ:idx];
    }
    [self hidePickerView];
}
- (IBAction)linkProfilePickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    if (self.pickerView.tag == PICKER_VIEW_Link_Profile ) {
        [self setLinkProfile:idx];
        p252bManager.asReaderP252B.linkProfileValue = (int)self.linkProfileValue;
    } else if (self.pickerView.tag == PICKER_VIEW_Default_Link_Profile) {
        [self setDefaultLinkProfile:idx];
        [systemSetting setDefaultLinkProfileValue:(int)self.defaultLinkProfileValue];
    }
    [self hidePickerView];
}

- (IBAction)buzzerPickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    [self setBuzzer:idx];
    [self hidePickerView];
}

- (IBAction)keyActionPickerOkBtnTapped:(id)sender {
    NSInteger idx = [self.pickerView selectedRowInComponent:0];
    [self setKeyAction:idx];
    [self hidePickerView];
}

- (IBAction)onClickBaudRateOKButton:(UIButton *)sender {
    self.baudRateValue = [self.pickerView selectedRowInComponent:0];
    [self setBaudRate:[self changeBaudRate:self.baudRateValue]];
    [systemSetting setBaudRate:(int)self.baudRateValue];
    [self hidePickerView];
}

- (IBAction)maskTypePickerOkBtnTapped:(id)sender {
    
}
- (IBAction)lbtChannelBtnTapped:(id)sender {
//    [AppDelegate showGlobalProgressHUDWithTitle:@"LBT Channel report"];
//    [CATransaction commit];
//    [self initLbtItemData];
//    [AppDelegate dismissGlobalHUD];
    [self.tableView reloadData];
    self.lbtChannelTableTopView.hidden = NO;
    self.tableView.hidden = NO;
    self.shadowView.hidden = NO;
}

- (IBAction)tableCloseBtnTapped:(id)sender {
    [self hideTableView];
}

- (IBAction)lbtChannelTableOkBtnTapped:(id)sender {
    [self hideTableView];
    [p252bManager.asReaderP252B setFrequency:self.lbtItemData];
    [systemSetting setLbtItemData:self.lbtItemData];
}
- (IBAction)onClickBarcodeTimeoutButton:(UIButton *)sender {
    UIAlertView *alert = [[UIAlertView alloc] initWithTitle:@"Barcode Timeout:" message:@"Please enter barcode timeout.\n(4~300s)" delegate:self cancelButtonTitle:@"Cancel" otherButtonTitles:nil];
    alert.alertViewStyle = UIAlertViewStylePlainTextInput;
    [alert textFieldAtIndex:0].keyboardType = UIKeyboardTypeNumberPad;
    alert.tag = ALERT_VIEW_BARCODE_TIMEOUT;
    [alert addButtonWithTitle:@"OK"];
    [alert show];
}
- (IBAction)onClickBaudRateButton:(UIButton *)sender {
    self.pickerView.tag = PICKER_VIEW_Baud_Rate;
    [self.pickerView reloadAllComponents];
    if (![self.baudRateTextField.text isEqualToString:@""]) {
        [self.pickerView selectRow:[self.mBaudRateList indexOfObject:self.baudRateTextField.text] inComponent:0 animated:NO];
    } else {
        [self.pickerView selectRow:0 inComponent:0 animated:NO];
    }
    self.baudRateTopView.hidden = NO;
    [self showPickerView];
}

#pragma mark - Custom
- (void)showPickerView {
    self.pickerView.hidden = NO;
    self.shadowView.hidden = NO;
}
- (void)hidePickerView {
    self.LinkProfileTableTopView.hidden = YES;
    self.qValuePickerTopView.hidden = YES;
    self.buzzerPickerTopView.hidden = YES;
    self.keyActionPickerTopView.hidden = YES;
    self.maskTypePickerTopView.hidden = YES;
    self.baudRateTopView.hidden = YES;
    self.pickerView.hidden = YES;
    self.shadowView.hidden = YES;
}
- (void)hideTableView {
    self.lbtChannelTableTopView.hidden = YES;
    self.tableView.hidden = YES;
    self.shadowView.hidden = YES;
}
- (void)setBuzzer:(NSInteger)index {
    self.buzzerTextField.text = [self.buzzerPickerData objectAtIndex:index];
    self.buzzerValue = (int)index;
}
- (void)setMinQ:(NSInteger)index {
    if (index > self.maxQValue) {
        index = self.maxQValue;
    }
    self.minQValueTextField.text = [self.qValuePickerData objectAtIndex:index];
    self.minQValue = (int)index;
}
- (void)setMaxQ:(NSInteger)index {
    if (index < self.minQValue) {
        index = self.minQValue;
    }
    self.maxQValueTextField.text = [self.qValuePickerData objectAtIndex:index];
    self.maxQValue = (int)index;
}

- (void)setLinkProfile:(NSInteger)index{
    self.linkProfileTextField.text = [self.linkProfilePickerData objectAtIndex:index];
    self.linkProfileValue = index;
}

- (void)setDefaultLinkProfile:(NSInteger)index{
    self.defaultLinkProfileTextField.text = [self.linkProfilePickerData objectAtIndex:index];
    self.defaultLinkProfileValue = index;
}

- (void)setQ:(NSInteger)index {
    self.qValueTextField.text = [self.qValuePickerData objectAtIndex:index];
    self.qValue = (int)index;
}
- (void)setIdleTime:(NSString *)idleTime {
    self.idleTimeTextField.text = idleTime;
    self.idleTimeValue = [idleTime integerValue];
}
- (void)setAutoOffTime:(NSString *)autoOffTime {
    self.autoOffTimeTextField.text = autoOffTime;
    self.autoOffTimeValue = [autoOffTime integerValue];
}
- (void)setKeyAction:(NSInteger)index {
    self.keyActionTextField.text = [self.keyActionPickerData objectAtIndex:index];
    self.keyActionValue = index;
    
}
- (void)setBatteryInterval:(NSString *)batteryInterval {
    self.batteryIntervalTextField.text = batteryInterval;
    self.batteryIntervalValue = batteryInterval;
}
- (void)setSleepTime:(NSString *)sleepTime {
    self.sleepTimeTextField.text = sleepTime;
    self.sleepTimeValue = [sleepTime integerValue];
}
- (void)setBarcodeTimeout:(NSString *)timeout {
    self.barcodeTimeoutTextField.text = timeout;
    self.barcodeTimeoutValue = [timeout integerValue];
}
- (void)setBaudRate:(NSString *)baudRate {
    self.baudRateTextField.text = baudRate;
}
- (void)initViewData {
    [self setMaxQ:self.maxQValue];
    [self setMinQ:self.minQValue];
    [self setLinkProfile:self.linkProfileValue];
    [self setDefaultLinkProfile:self.defaultLinkProfileValue];
    self.linkProfileTextField.text  = [NSString stringWithFormat:@"%ld",(long)self.linkProfileValue];
    [self setQ:self.qValue];
    [self setBuzzer:self.buzzerValue];
    [self setIdleTime:[NSString stringWithFormat:@"%ld", (long)self.idleTimeValue]];
    [self setAutoOffTime:[NSString stringWithFormat:@"%ld", (long)self.autoOffTimeValue]];
    [self setSleepTime:[NSString stringWithFormat:@"%ld", (long)self.sleepTimeValue]];
    [self setBarcodeTimeout:[NSString stringWithFormat:@"%ld", (long)self.barcodeTimeoutValue]];
    [self setBaudRate:[self changeBaudRate:self.baudRateValue]];
    NSString *batteryInterval = [[SystemSetting sharedSystemSetting] getBatteryInterval];
    [self setBatteryInterval:batteryInterval];
}
- (NSString *)changeBaudRate:(NSInteger)index {
    if (self.mBaudRateList.count <= 0) {
        return @"115200 bps";
    }
    return [self.mBaudRateList objectAtIndex:index];
}
#pragma mark - NSNotification 673834852
#pragma mark - Reader Event Handler Methods
- (BOOL)onAsReaderP252BTriggerKeyEvent:(BOOL)status{
    return NO;
}
- (void)updateAsReaderP252BDeviceState:(AsReaderP252BResultCode)error {
    if (error != AsReaderP252BResultNoError) {
        UIAlertView *alert = [[UIAlertView alloc]initWithTitle:@"Error" message:@"Failed to get response. Try again." delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
        [alert show];
        
        [self.navigationController popViewControllerAnimated:YES];
    }
}

-(void)whenAsReaderP252BIsConnected:(BOOL)isConnected{
    if (!isConnected) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [self hideTableView];
            [self hidePickerView];
            [self.navigationController popToRootViewControllerAnimated:YES];
        });
    }
}
@end
