//
//  RFIDFirmwareManager.m
//  AsReaderP252BDemo
//
//  Created by Songxueqian on 2022/4/1.
//  Copyright © 2022 asterisk. All rights reserved.
//

#import "RFIDFirmwareManager.h"

#define PACKET_PREFIX            0x7e          // '~'
#define PACKET_SUFFIX1           0x0d          // '\r'
#define PACKET_SUFFIX2           0x0a          // '\n'
#define PACKET                   256.0
#define STARTDATA                @"5A0001040100060000000000003F7A"
#define PICKETDATA               @"5A00010401"
#define PICKETINFODATA           @"00010401"
#define ENDDATA                  @"5A000104010006FFFFFFFF0000B5AA"
#define RESETDATA                @"5A0001010F0000F0D4"

@interface RFIDFirmwareManager ()
{
    NSMutableData *_mRecvData;
    BOOL updateFirmwareStatus;
    AsReaderP252BPacketManager * mPacket;
    AsReaderP252BManager *p252bManager;
}
@end
@implementation RFIDFirmwareManager
static RFIDFirmwareManager * sharedRFIDFirmwareManager;
+ (RFIDFirmwareManager *)sharedRFIDFirmwareManager{
    if (!sharedRFIDFirmwareManager) {
        sharedRFIDFirmwareManager = [[RFIDFirmwareManager alloc] init];
    }
    return sharedRFIDFirmwareManager;
}
- (instancetype)init
{
    self = [super init];
    if (self) {
        updateFirmwareStatus = NO;
        mPacket = [[AsReaderP252BPacketManager alloc] init];
        p252bManager = [AsReaderP252BManager sharedAsReaderP252BManager];
    }
    return self;
}
- (void)updateRFIDFirmwareWithFileName:(NSString *)fileName
                          fileData:(NSData *)data
                            result:(void(^)(BOOL result))result
                          progress:(void(^)(NSProgress * progress))progress{
    @try {
        if (!data || data.length == 0 || data.length % 2 != 0) {
            result(NO);
            return;
        }
        
        if (!p252bManager.asReaderP252B.isConnected) {
            result(NO);
            return;
        }
        
        [self startUpdateFirmware];
        
        NSString *fileSize = 0;
        fileSize = [NSString stringWithFormat:@"%lu",(unsigned long)data.length];
        int countPicket =  ceilf(data.length/PACKET);
        NSLog(@"countPicket : %d",countPicket);
        NSProgress * prog = [NSProgress progressWithTotalUnitCount:countPicket];

        //请求固件更新
        NSData *startData = [self convertHexStrToData:STARTDATA];
        BOOL updateFirmwareInfo = [self writeFirmwareData:startData Property:P252BFirmwareTypePicketStart];
        if (updateFirmwareInfo == NO) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        for (int i = 0 ; i < countPicket; i++) {
            NSData *sendData = nil;
            if (data.length > i*PACKET + PACKET) {
                sendData = [data subdataWithRange:NSMakeRange( i*PACKET, PACKET)];
            }else{
                sendData = [data subdataWithRange:NSMakeRange( i*PACKET, data.length - i*PACKET)];
            }
            if (!p252bManager.asReaderP252B.isConnected) {
                NSLog(@"_W3 is not connected");
                result(NO);
                [self stopUpdateFirmware];
                return;
            }
            
            NSMutableData *sendPicketData = [[NSMutableData alloc] init];
            Byte picket[4] = {0};
            picket[0] = (i + 1&0xFF000000)>>24;
            picket[1] = (i + 1&0x00FF0000)>>16;
            picket[2] = (i + 1&0x0000FF00)>>8;
            picket[3] = (i + 1&0x000000FF);
            [sendPicketData appendData:[NSData dataWithBytes:picket length:4]];
            
            Byte picketLength[2] = {0};
            picketLength[0] = (sendData.length&0xFF00)>>8;
            picketLength[1] = (sendData.length&0x00FF);
            [sendPicketData appendData:[NSData dataWithBytes:picketLength length:2]];
            [sendPicketData appendData:sendData];
            
            NSMutableData *newData = [[NSMutableData alloc] init];
            NSData *picketData = [self convertHexStrToData:PICKETDATA];
            NSData *lenghtData = [self convertHexStrToData:[NSString stringWithFormat:@"%04x", (int)sendPicketData.length]];
            [newData appendData:picketData];
            [newData appendData:lenghtData];
            [newData appendData:sendPicketData];
            
            NSMutableData *crcAndData = [[NSMutableData alloc] init];
            [crcAndData appendData:[self convertHexStrToData:PICKETINFODATA]];
            [crcAndData appendData:lenghtData];
            [crcAndData appendData:sendPicketData];
            int crc = [self calculateCRC:crcAndData];
            NSData *crcData = [self convertHexStrToData:[NSString stringWithFormat:@"%04x", crc]];
            [newData appendData:crcData];
            //发送bin文件数据
            BOOL updateFirmwarePickety = [self writeFirmwareData:newData Property:P252BFirmwareTypePicketData];
            [NSThread sleepForTimeInterval:0.05];
            if (updateFirmwarePickety == NO) {
                result(NO);
                [self stopUpdateFirmware];
                return;
            } else {
                prog.completedUnitCount++;
                progress(prog);
            }
        }
        //固件更新完成
        NSData *endData = [self convertHexStrToData:ENDDATA];
        BOOL updateFirmwareEND = [self writeFirmwareData:endData Property:P252BFirmwareTypePicketEND];
        [NSThread sleepForTimeInterval:0.05];
        if (updateFirmwareEND == NO) {
            result(NO);
            [self stopUpdateFirmware];
            return;
        }
        //重启RFID模组
        NSData *resetData = [self convertHexStrToData:RESETDATA];
        [self writeFirmwareData:resetData Property:P252BFirmwareTypePicketReset];
        [prog setCompletedUnitCount:countPicket];
        progress(prog);
        result(YES);
        [self stopUpdateFirmware];
    } @catch (NSException *exception) {
        result(NO);
        [self stopUpdateFirmware];
    } @finally {
    }
}
- (void)rX:(NSData *)readData {
    if (updateFirmwareStatus) {
        if (readData && readData.length > 0) {
            [self readData:readData];
        }
    }
}
- (BOOL)tX:(NSData *)transmitData {
    if (!updateFirmwareStatus) {
        return NO;
    }
    [p252bManager.asReaderP252B tx:transmitData];
    return YES;
}
- (void)startUpdateFirmware {
    updateFirmwareStatus = YES;
}
- (void)stopUpdateFirmware {
    updateFirmwareStatus = NO;
}
- (BOOL)writeFirmwareData:(NSData *)data Property:(P252BFirmwareType)property {
    _mRecvData = [[NSMutableData alloc] init];
    NSString *propertyString = [NSString stringWithFormat:@"~w%c", property];
    NSString *endData = @"\r\n";
    NSMutableData *sendData = [[NSMutableData alloc] initWithData:[propertyString dataUsingEncoding:NSASCIIStringEncoding]];
    if (data) {
        [sendData appendData:data];
    }
    [sendData appendData:[endData dataUsingEncoding:NSASCIIStringEncoding]];
    BOOL tXResult = [self tX:sendData];
    if (!tXResult) {
        return NO;
    }
    AsReaderP252BPacket *packet = [[AsReaderP252BPacket alloc]initWithPacketType:AsReaderP252BPacketResSetProperty subType:property];

    [self wite:packet];
    AsReaderP252BResultCode resCode = (AsReaderP252BResultCode)packet.errorCode;
    if (resCode != AsReaderP252BResultNoError) {
        return NO;
    } else {
        return YES;
    }
}
- (void)wite:(AsReaderP252BPacket *)packet{
    [mPacket addPacket:packet];
    [NSThread detachNewThreadSelector:@selector(changePacket:) toTarget:self withObject:packet];
    while (!packet.data) {
        if (!p252bManager.asReaderP252B.isConnected) {
            break;
        }
        [NSThread sleepForTimeInterval:0.01];
        [[NSRunLoop currentRunLoop] runMode:NSDefaultRunLoopMode beforeDate:[NSDate distantFuture]];
    }
    [NSObject cancelPreviousPerformRequestsWithTarget:self selector:@selector(changePacket:) object:packet];
    [mPacket removePacket:packet];

}

- (void)changePacket:(AsReaderP252BPacket *)packet{
    [NSThread sleepForTimeInterval:2.0];
    [self performSelectorOnMainThread:@selector(cPacket:) withObject:packet waitUntilDone:NO];
}

- (void)cPacket:(AsReaderP252BPacket *)packet{
    if (!packet.data) {
        packet.data = [[NSData alloc] init];
        packet.errorCode = AsReaderP252BResultOtherError;
    }
}
- (void)readData:(NSData *)data {
    [_mRecvData appendData:data];
    NSData *recvData = nil;
    NSRange range;
    while(true)
    {
        range = [AsReaderP252BDataUtil findDataWidthRange:_mRecvData withPrefix:PACKET_PREFIX withSuffix1:PACKET_SUFFIX1 withSuffix2:PACKET_SUFFIX2];
        if (range.location == NSNotFound && range.length == NSNotFound) {
            return;
        }
        if (range.location != NSNotFound && range.length != NSNotFound) {
            if (range.length < 6) {
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
                return;
            } else {
                recvData = [_mRecvData subdataWithRange:NSMakeRange(range.location + 1, range.length - 3)];
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            }
        } else if (range.location != NSNotFound && range.length == NSNotFound) {
            if (range.location > 0) {
                range.length = range.location - 1;
                range.location = 0;
                [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            }
            return;
        } else if (range.location == NSNotFound && range.length != NSNotFound) {
            range.location = 0;
            [_mRecvData replaceBytesInRange:range withBytes:NULL length:0];
            return;
        } else {
            return;
        }

        if (recvData.length < 2) {
            return;
        }
        uint8_t *totalPack = (uint8_t *)[recvData bytes];
        uint8_t pack = totalPack[0];
        uint8_t subPack = totalPack[1];
        NSInteger errorCode = [self getErrorCode:recvData findPosition:2];
        NSData *value = nil;
        int offset = 2;
        if (recvData.length > 2) {
            value = [recvData subdataWithRange:NSMakeRange(offset, recvData.length - offset)];
        }
        switch (pack) {
            case P252BFirmwarePropertyTypeGet:
                break;
            case P252BFirmwarePropertyTypeSet:
                switch (subPack) {
                    case P252BFirmwareTypePicketStart:
                    case P252BFirmwareTypePicketData:
                    case P252BFirmwareTypePicketEND:
                    {
                        AsReaderP252BPacket *packet = [mPacket findPacket:AsReaderP252BPacketResSetProperty subType:subPack];
                        if (packet != nil) {
                            packet.errorCode = errorCode;
                            packet.data = value;
                            [packet signal];
                        }
                    }
                        break;
                    default:
                       
                        break;
                }
                break;
            default:
                break;
               
        }
    }
}
- (NSInteger)getErrorCode:(NSData *)data findPosition:(int)position {
    int length = 4;
    if ( (int)[data length] - position < 4) {
        length = (int)[data length] - position;
    }
    NSRange range = NSMakeRange(position, length);
    NSData *value = [data subdataWithRange:range];
    NSString *convValue = [[NSString alloc]initWithData:value encoding:NSASCIIStringEncoding];
    unsigned res = 0;
    [[NSScanner scannerWithString:convValue]scanHexInt:&res];
    return res;
}
- (NSData *)convertHexStrToData:(NSString *)str {
    if (!str || [str length] == 0) {
        return nil;
    }
    NSMutableData *hexData = [[NSMutableData alloc] initWithCapacity:8];
    NSRange range;
    if ([str length] % 2 == 0) {
        range = NSMakeRange(0, 2);
    } else {
        range = NSMakeRange(0, 1);
    }
    for (NSInteger i = range.location; i < [str length]; i += 2) {
        unsigned int anInt;
        NSString *hexCharStr = [str substringWithRange:range];
        NSScanner *scanner = [[NSScanner alloc] initWithString:hexCharStr];
        [scanner scanHexInt:&anInt];
        NSData *entity = [[NSData alloc] initWithBytes:&anInt length:1];
        [hexData appendData:entity];
        range.location += range.length;
        range.length = 2;
    }
    return hexData;
}

- (unsigned short)calculateCRC:(NSData *)packetData {
    unsigned short CRC_16_CCITT = 0x1021;
    unsigned char chChar;
    unsigned short wCRC = 0;
    Byte *packet = (Byte *)[packetData bytes];
    for (short i = 0; i < packetData.length; i++) {
        chChar = (char) packet[i];
        wCRC ^= (((short)chChar) << 8);
        for (int j = 0; j < 8; j++)
        {
            int  a = wCRC & 0x8000;
            if ( a != 0)
                wCRC = (short)((wCRC << 1) ^ CRC_16_CCITT);
            else
                wCRC <<= 1;
        }
    }
    return wCRC;
}
@end
